<?php
/**
* Mosets Tree
*
* This extension will give the SEF advance style URLs to Mosets Tree
*
* Updated on 2015-02-11, tested with Mosets Tree v3.5.9, Joomla v3.3.6
*
* Copyright (C) 2008-2015 Emir Sakic, http://www.sakic.net, All rights reserved.
*
* @package Mosets Tree
* @copyright (C) 2005-2007 Mosets Consulting
* @url http://www.mosets.com/
* @author Lee Cher Yeong <mtree@mosets.com> **/

require( dirname(realpath(__FILE__)) . '/init.php');
require_once( dirname(realpath(__FILE__)) . '/../../administrator/components/com_mtree/admin.mtree.class.php' );

define("_MT_SEF_ADD_LINKID", 0);		// Setting this to 1 will add listing ID number as a prefix to the listing name in the URL. This is used in cases where a category has more than one listing with the same name.
define("_MT_SEF_ADD_LINKID_SEP", "_");	// Separator to separate Listing ID and Listing Name
define("_MT_SEF_ADD_LINKID_ATTACH", 1);	// (1) Add Link ID as a suffix; (2)  Add Link ID as a prefix
define("_MT_SEF_USE_ALIAS", 0);			// Setting this to 1 will use alias for URL rather than title

define("_MT_SEF_IMAGE", "image");
define("_MT_SEF_GALLERY", "gallery");
define("_MT_SEF_DETAILS", "details");
define("_MT_SEF_REVIEW", "review");
define("_MT_SEF_REPLYREVIEW", "replyreview");
define("_MT_SEF_REPORTREVIEW", "reportreview");
define("_MT_SEF_RATE", "rate");
define("_MT_SEF_RECOMMEND", "recommend");
define("_MT_SEF_CONTACT", "contact");
define("_MT_SEF_REPORT", "report");
define("_MT_SEF_CLAIM", "claim");
define("_MT_SEF_VISIT", "visit");
define("_MT_SEF_CATEGORY_PAGE", "page");
define("_MT_SEF_DELETE", "delete");
define("_MT_SEF_EDITLISTING","edit");
define("_MT_SEF_REVIEWS_PAGE","reviews");
define("_MT_SEF_ADDLISTING","Add_Listing");
define("_MT_SEF_ADDCATEGORY","Add_Category");
define("_MT_SEF_LISTINGTYPE","Listing_Type");

define("_MT_SEF_MYPAGE","mypage");
define("_MT_SEF_NEWLISTING","New_Listing");
define("_MT_SEF_RECENTLYUPDATED","Recently_Updated");
define("_MT_SEF_MOSTFAVOURED","Most_Favoured");
define("_MT_SEF_FEATUREDLISTING","Featured_Listing");
define("_MT_SEF_POPULARLISTING","Popular_Listing");
define("_MT_SEF_VERIFIEDLISTING","Verified_Listing");
define("_MT_SEF_MOSTRATEDLISTING","Most_Rated");
define("_MT_SEF_TOPRATEDLISTING","Top_Rated");
define("_MT_SEF_MOSTREVIEWEDLISTING","Most_Reviewed");
define("_MT_SEF_LISTALPHA","List_Alpha");

define("_MT_SEF_OWNER", "Owner");
define("_MT_SEF_FAVOURITES", "Favourites");
define("_MT_SEF_REVIEWS", "Reviews");
define("_MT_SEF_SEARCH", "Search");
define("_MT_SEF_ADVSEARCH", "AdvSearch");
define("_MT_SEF_ADVSEARCH2", "AdvSearchR");

define("_MT_SEF_SEARCHBY", "Search_By");

class sef_mtree {

	var $querystring=null;

	/********************************************************
	* CREATE
	********************************************************/

	/**
	* Creates the SEF advance URL out of the Joomla! request
	* Input: $string, string, The request URL (index.php?option=com_mtree&Itemid=$Itemid)
	* Output: $sefstring, string, SEF advance URL ($var1/$var2/)
	**/
	function create ($string) {
		global $mtconf;
		
		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}
		
		$sefstring = '';
		
		# print
		if (eregi("&amp;tmpl=component",$string)) {
			return str_replace('&amp;', '&', $string);
		}

		# List Categories (listcats)
		if ( eregi("&amp;task=listcats&amp;cat_id=",$string) || eregi("&amp;view=listcats&amp;cat_id=",$string) ) {

			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
	
			//if( $this->getLimits( $limit, $limitstart, $string ) ) {
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= _MT_SEF_CATEGORY_PAGE . $page . '/';
			}

		}
		
		# RSS
		if ( eregi("&amp;task=rss",$string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= 'rss/';
			if (eregi("&amp;type=updated",$string)) {
				$sefstring .= 'updated/';
			}
		}

		# My Page
		if ( $this->taskIs("mypage", $string) ) {

			$sefstring .= _MT_SEF_MYPAGE .'/';

			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= _MT_SEF_CATEGORY_PAGE . $page . '/';
			}

		}

		# Featured Listing
		if ( $this->taskIs("listfeatured", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_FEATUREDLISTING . '/';

			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
			
		}
		
		# Verified Listing
		if ( $this->taskIs("listverified",$string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_VERIFIEDLISTING . '/';
			
			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
			
		}

		# New/Latest Listing
		if ( $this->taskIs("listnew", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_NEWLISTING . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = ceil(($limitstart / $mtconf->get('fe_num_of_links')) + 1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}

		}

		# Recently Updated Listing
		if ( $this->taskIs("listupdated", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_RECENTLYUPDATED . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
		}

		# Most Favoured Listing
		if ( $this->taskIs("listfavourite", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_MOSTFAVOURED . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
		}
		
		# Popular Listing
		if ( $this->taskIs("listpopular", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_POPULARLISTING . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
		}

		# Most Rated Listing
		if ( $this->taskIs("listmostrated", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_MOSTRATEDLISTING . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
		}

		# Top Rated Listing
		if ( $this->taskIs("listtoprated", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_TOPRATEDLISTING . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
		}

		# Most Reviewed Listing
		if ( $this->taskIs("listmostreview", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );
			$sefstring .= _MT_SEF_MOSTREVIEWEDLISTING . '/';
			
			if( preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {
				$limitstart = !empty($matches[1]) ? $matches[1] : 0;
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}
		}

		# List Alpha
		if ( $this->taskIs("listalpha", $string) ) {
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );

			$sefstring .= _MT_SEF_LISTALPHA . '/';

			// Get start alphabet
			$temp = split("&amp;alpha=", $string);
			$temp = split("&", $temp[1]);
			$sefstring .= urlencode($temp[0]) . '/';

			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				//	http://example.com/c/mtree/Computer/Games/List_Alpha/d/page25.html
				
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			
			} else {
				$sefstring .= '1' . '/';

			}

		}

		# Advanced Search Results
		if ( $this->taskIs("advsearch2", $string) ) {
			$sefstring .= _MT_SEF_ADVSEARCH2 . '/';

			// Get search id
			$search_id = $this->getID("search",$string);
			$page = 1;

			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				$page = (($limitstart / $mtconf->get('fe_num_of_searchresults')) +1);
			}

			//if ( $page <> 1 && $page <> 0 ) {
			if ( $page > 0 ) {
				$sefstring .= $page . '/' . $search_id .'/';
			}

		# Advanced Search
		// Note: This has to be places in the else statement, otherwise, advsearch2 task 
		//			 will append _MT_SEF_ADVSEARCH another time at the end of the URL
		} elseif ( $this->taskIs("advsearch", $string) ) {
			$sefstring .= _MT_SEF_ADVSEARCH . '/';
		}
		
		# View All listing from Owner
		if ( $this->taskIs("viewowner", $string) || $this->taskIs("viewusersreview", $string) || $this->taskIs("viewusersfav", $string) ) {

			$user_id = $this->getID( 'user', $string );
			$db->setQuery( "SELECT username FROM #__users WHERE id='".$user_id."' AND block='0'" );
			$username = $db->loadResult();

			if ( !empty($username) ) {
				if( $this->taskIs("viewowner", $string) ) {
					$sefstring .= _MT_SEF_OWNER;
				} elseif( $this->taskIs("viewusersreview", $string) ) {
					$sefstring .= _MT_SEF_REVIEWS;
				} else {
					$sefstring .= _MT_SEF_FAVOURITES;
				}
				$sefstring .= '/' . sefencode($username) . '/';
			}

			// TODO - Does not append further virtual path if username does not exists. mtree.php
			//			 should check if user is not block / exists.
			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				$page = (($limitstart / $mtconf->get('fe_num_of_links')) +1);
				$sefstring .= $page . '/';
			} else {
				$sefstring .= '1' . '/';
			}

		}
		
		# View Image
		if ( $this->taskIs("viewimage",$string) ) {
			$sefstring .= _MT_SEF_IMAGE . '/' . $this->getID('img',$string) . '/';
		}

		# View Listing
		if ( $this->listingTaskIs("viewlink",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCat( $mtLink->cat_id );
			
			if ( _MT_SEF_USE_ALIAS ) {
				$link_name = $mtLink->alias;
			} else {
				$link_name = $mtLink->link_name;
			}

			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				//	http://example.com/c/mtree/Computer/Games/Donkey_Kong/reviews23/
				$page = (($limitstart / $mtconf->get('fe_num_of_reviews')) +1);
				$sefstring .= $this->appendListing( $link_name, $mtLink->link_id, false );
				$sefstring .= _MT_SEF_REVIEWS_PAGE . $page . '/';
			} else {
				//	http://example.com/c/mtree/Computer/Games/Donkey_Kong/details/
				$sefstring .= $this->appendListing( $link_name, $mtLink->link_id, true );
			}
		}

		# View Gallery
		if ( $this->listingTaskIs("viewgallery",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_GALLERY . '/';
		}
		
		# Reply Review
		if ( $this->taskIs("replyreview", $string) ) {
			$sefstring .= _MT_SEF_REPLYREVIEW . '/' . $this->getID( 'rev', $string ) . '/';
		}

		# Report Review
		if ( $this->taskIs("reportreview", $string) ) {
			$sefstring .= _MT_SEF_REPORTREVIEW . '/' . $this->getID( 'rev', $string ) . '/';
		}

		# Write Review
		if ( $this->listingTaskIs("writereview",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_REVIEW . '/';
		}

		# Rating
		if ( $this->listingTaskIs("rate",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_RATE . '/';
		}

		# RECOMMEND
		if ( $this->listingTaskIs("recommend",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_RECOMMEND . '/';
		}

		# CONTACT OWNER
		if ( $this->listingTaskIs("contact",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_CONTACT . '/';
		}

		# REPORT LISTING
		if ( $this->listingTaskIs("report",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_REPORT . '/';
		}

		# CLAIM LISTING
		if ( $this->listingTaskIs("claim",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_CLAIM . '/';
		}

		# VISIT LISTING
		if ( $this->listingTaskIs("visit",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_VISIT . '/';
		}
		
		# DELETE LISTING
		if ( $this->listingTaskIs("deletelisting",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_DELETE . '/';
		}
		
		# EDIT LISTING
		if ( $this->listingTaskIs("editlisting",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_EDITLISTING . '/';
		}
		
		# Add Listing
		if ( $this->listingTaskIs("addlisting",$string) || eregi("&amp;task=addlisting&",$string) ) {

			if (eregi("&amp;task=addlisting&amp;link_id=",$string)) {
				
				$mtLink = new mtLinks( $db );
				$mtLink->load( $this->getID('link',$string) );
				$sefstring .= $this->appendCat( $mtLink->cat_id );

			} elseif (eregi("&amp;task=addlisting&amp;cat_id=",$string)) {
				
				$cat_id = $this->getID( 'cat', $string );
				$sefstring .= $this->appendCat( $cat_id );

			}

			$sefstring .= _MT_SEF_ADDLISTING . '/';

		}
		
		# LISTING TYPE
		if ( $this->listingTaskIs("listingtype",$string) ) {
			$mtLink = new mtLinks( $db );
			$mtLink->load( $this->getID('link',$string) );
			$sefstring .= $this->appendCatListing( $mtLink, false );
			$sefstring .= _MT_SEF_LISTINGTYPE . '/';
		}

		# Add Category
		if ( $this->listingTaskIs("addcategory",$string) || eregi("&amp;task=addcategory",$string) ) {

			if (eregi("&amp;task=addcategory&amp;link_id=",$string)) {
				
				$mtLink = new mtLinks( $db );
				$mtLink->load( $this->getID('link',$string) );
				$sefstring .= $this->appendCat( $mtLink->cat_id );

			} elseif (eregi("&amp;task=addcategory&amp;cat_id=",$string)) {
				
				$cat_id = $this->getID( 'cat', $string );
				$sefstring .= $this->appendCat( $cat_id );

			}

			$sefstring .= _MT_SEF_ADDCATEGORY . '/';

		}

		# Search Results
		if ( $this->taskIs("searchby", $string) ) {

			$sefstring .= _MT_SEF_SEARCHBY . '/';
			
			/*
			// Get tag alias
			$temp = split("&amp;cf_id=", $string);
			$temp = split("&", $temp[1]);
			$cf_id = $temp[0];
			$alias = $this->appendTag($cf_id);

			$sefstring .= sefencode($alias) . '/';

			// Get tag value
			$temp = split("&amp;value=", $string);
			$temp = split("&", $temp[1]);
			$value = $temp[0];

			$sefstring .= sefencode(urldecode($value)) . '/';
			
			$sefstring .= $this->append_ordering_and_limits($string);
			
			$sefstring .= '?';
			*/
			
			$temp = split("&amp;cf_id=", $string);
			$sefstring .= '?cf_id='.$temp[1];			

		} else if ( $this->taskIs("search",$string) ) {
			
			$cat_id = $this->getID( 'cat', $string );
			$sefstring .= $this->appendCat( $cat_id );

			$sefstring .= _MT_SEF_SEARCH . '/';

			// Get search word
			$temp = split("&amp;searchword=", $string);
			$temp = split("&", $temp[1]);
			$searchword = $temp[0];
			$page = 1;

			if( $this->getLimits( $limit, $limitstart, $string ) ) {
				$page = (($limitstart / $mtconf->get('fe_num_of_searchresults')) +1);
			}

			$sefstring .= $page . '/' . $searchword . '/';

		}
		
		# Pagination only
		if ( !stristr($string, 'task=') && preg_match('/limitstart=([0-9]*)/', $string, $matches) ) {

			$task = $_REQUEST['task'];
			switch ($task) {
				case 'listnew':
					$sefstring .= _MT_SEF_NEWLISTING . '/';
					$limit = 'fe_num_of_new';
					break;
				case 'listupdated':
					$sefstring .= _MT_SEF_RECENTLYUPDATED . '/';
					$limit = 'fe_num_of_updated';
					break;
				case 'listfavourite':
					$sefstring .= _MT_SEF_MOSTFAVOURED . '/';
					$limit = 'fe_num_of_favourite';
					break;
				case 'listpopular':
					$sefstring .= _MT_SEF_POPULARLISTING . '/';
					$limit = 'fe_num_of_popular';
					break;
				case 'listmostrated':
					$sefstring .= _MT_SEF_MOSTRATEDLISTING . '/';
					$limit = 'fe_num_of_mostrated';
					break;
				case 'listtoprated':
					$sefstring .= _MT_SEF_TOPRATEDLISTING . '/';
					$limit = 'fe_num_of_toprated';
					break;
				case 'listmostreview':
					$sefstring .= _MT_SEF_MOSTREVIEWEDLISTING . '/';
					$limit = 'fe_num_of_reviews';
					break;
				default:
					$limit = 'fe_num_of_links';
					break;
			}
			
			$limitstart = !empty($matches[1]) ? $matches[1] : 0;
			$page = ceil(($limitstart / $mtconf->get($limit)) + 1);
			$sefstring .= $page . '/';

		}
		
		if (_MT_SEF_USE_ALIAS) {
			$sefstring = str_replace('-', '~', $sefstring);
		}

		return $sefstring;

	}

	/********************************************************
	* REVERT
	********************************************************/

	/**
	* Reverts to the Joomla! query string out of the SEF advance URL
	* Input:
	*    $url_array, array, The SEF advance URL split in arrays (first custom virtual directory beginning at $pos+1)
	*    $pos, int, The position of the first virtual directory (component)
	* Output: $QUERY_STRING, string, Joomla! query string (var1=$var1&var2=$var2)
	*    Note that this will be added to already defined first part (option=com_example&Itemid=$Itemid)
	**/
	function revert ($url_array, $pos) {
		global $mtconf;
		
		if (_MT_SEF_USE_ALIAS) {
			$i = 0;
			foreach ($url_array as $part) {
				$url_array[$i] = str_replace('~', '-', $url_array[$i]);
				$i++;
			}
		}
		
		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}
		
		if(!isset($mtconf)) {
			$mtconf	= new mtConfig($db);
		}

		$QUERY_STRING = "";
		
		if ( (isset($url_array[$pos+2]) && $url_array[$pos+2]!='') ) {

			# (1) Viewing Category	- http://example.com/c/mtree/Computer/Games/
			#						- http://example.com/c/mtree/Computer/Games/page[0-9]+.html
			
			$pagepattern = _MT_SEF_CATEGORY_PAGE . "[0-9]+";
			
			if ( 
						(
							empty( $url_array[ (count($url_array)-1) ] ) 
							|| 
							( eregi($pagepattern,$url_array[ (count($url_array)-2) ]) ) 
						)
						&&
							( !in_array( $url_array[ (count($url_array)-3) ], array( _MT_SEF_FEATUREDLISTING, _MT_SEF_NEWLISTING, _MT_SEF_REPLYREVIEW, _MT_SEF_REPORTREVIEW, _MT_SEF_IMAGE, _MT_SEF_VERIFIEDLISTING ) ) )
						&&
							( !in_array( $url_array[ (count($url_array)-4) ], array( _MT_SEF_OWNER, _MT_SEF_REVIEWS, _MT_SEF_FAVOURITES, _MT_SEF_SEARCH, _MT_SEF_ADVSEARCH2, _MT_SEF_LISTALPHA, _MT_SEF_SEARCHBY ) ) )
						&&
							!in_array( $url_array[ (count($url_array)-2) ], array( _MT_SEF_REVIEW, _MT_SEF_RATE, _MT_SEF_RECOMMEND, _MT_SEF_CONTACT, _MT_SEF_REPORT, _MT_SEF_CLAIM, _MT_SEF_VISIT, _MT_SEF_DELETE, _MT_SEF_EDITLISTING, _MT_SEF_ADDLISTING, _MT_SEF_ADDCATEGORY, _MT_SEF_REVIEWS_PAGE, _MT_SEF_DETAILS, _MT_SEF_GALLERY, _MT_SEF_SEARCHBY, _MT_SEF_LISTINGTYPE ) )
						&&
							!eregi(_MT_SEF_REVIEWS_PAGE."[0-9]+",$url_array[ (count($url_array)-2) ])
						&&
							!in_array( $url_array[ $pos+2 ], array( _MT_SEF_RECENTLYUPDATED, _MT_SEF_MOSTFAVOURED, _MT_SEF_POPULARLISTING, _MT_SEF_MOSTRATEDLISTING, _MT_SEF_TOPRATEDLISTING, _MT_SEF_MOSTREVIEWEDLISTING, 'rss' ) )
				) {

				$rss = false;
				if ($url_array[ (count($url_array)-2) ]=='rss') {
					$rss = true;
					array_pop( $url_array );
				}
				
				$cat_names = array_slice( $url_array, ($pos+2), -1 );
				if ( eregi($pagepattern,$cat_names[count($cat_names)-1]) ) {
					array_pop( $cat_names );
				}
				$cat_id = $this->findCatID( $cat_names );

				switch( end($cat_names) ) {

					case _MT_SEF_MYPAGE:
						$QUERY_STRING .= $this->appendURL( 'task', 'mypage' );
						$pagenumber = $this->getPageNumber($url_array);
						
						if ( $pagenumber > 0 ) {
							$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_links') );
							$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($pagenumber -1)) );
						}
						break;

					case _MT_SEF_ADVSEARCH:
						$QUERY_STRING .= $this->appendURL( 'task', 'advsearch' );
						break;

					default:
						if ($rss) {
							$QUERY_STRING .= $this->appendURL( 'task', 'rss' );
						} else {
							$QUERY_STRING .= $this->appendURL( 'task', 'listcats' );
						}
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						
						$pagenumber = $this->getPageNumber($url_array);
						
						if ( $pagenumber > 0 ) {
							$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_links') );
							$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($pagenumber -1)) );
						}
						
						break;

				} // End switch()

			# (2) Viewing Listing 
			#		- http://example.com/c/mtree/Computer/Games/Donkey_Kong/details/
			#		- http://example.com/c/mtree/Computer/Games/Donkey_Kong/review1/

			
			//} elseif( $url_array[ (count($url_array)-2) ] == _MT_SEF_FEATUREDLISTING ) {
			} elseif( in_array( $url_array[ (count($url_array)-2) ], array( _MT_SEF_REVIEW, _MT_SEF_RATE, _MT_SEF_RECOMMEND, _MT_SEF_CONTACT, _MT_SEF_REPORT, _MT_SEF_CLAIM, _MT_SEF_VISIT, _MT_SEF_DELETE, _MT_SEF_EDITLISTING, _MT_SEF_ADDLISTING, _MT_SEF_ADDCATEGORY, _MT_SEF_REVIEWS_PAGE, _MT_SEF_DETAILS, _MT_SEF_GALLERY, _MT_SEF_LISTINGTYPE ) )
				|| in_array( $url_array[ $pos+2 ], array( _MT_SEF_RECENTLYUPDATED, _MT_SEF_MOSTFAVOURED, _MT_SEF_POPULARLISTING, _MT_SEF_MOSTRATEDLISTING, _MT_SEF_TOPRATEDLISTING, _MT_SEF_MOSTREVIEWEDLISTING ) )
				|| eregi(_MT_SEF_REVIEWS_PAGE."[0-9]+",$url_array[ (count($url_array)-2) ]) ) {

				// Get the virtual filename
				$vfilename = $url_array[ (count($url_array)-2) ];
				
				if (is_numeric($vfilename)) {
					$vfilename = $url_array[ $pos+2 ];
				}
				
				switch($vfilename) {
					
					case _MT_SEF_RECENTLYUPDATED:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listupdated' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;

					case _MT_SEF_MOSTFAVOURED:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listfavourite' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;

					case _MT_SEF_POPULARLISTING:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listpopular' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;

					case _MT_SEF_MOSTRATEDLISTING:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listmostrated' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;

					case _MT_SEF_TOPRATEDLISTING:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listtoprated' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;
						
					case _MT_SEF_MOSTREVIEWEDLISTING:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listmostreview' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;
						
					case _MT_SEF_REVIEW:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'writereview' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_RATE:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'rate' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_RECOMMEND:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'recommend' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_CONTACT:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'contact' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_REPORT:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'report' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_CLAIM:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'claim' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_VISIT:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'visit' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;
					
					case _MT_SEF_DELETE:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'deletelisting' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;
						
					case _MT_SEF_EDITLISTING:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'editlisting' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					case _MT_SEF_ADDLISTING:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'addlisting' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;

					case _MT_SEF_ADDCATEGORY:
						$cat_names = array_slice( $url_array, ($pos+2), -2 );
						$cat_id = $this->findCatID( $cat_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'addcategory' );
						$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
						break;

					case _MT_SEF_DETAILS:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'viewlink' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;
						
					case _MT_SEF_GALLERY:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'viewgallery' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;
						
					case _MT_SEF_LISTINGTYPE:
						$path_names = array_slice( $url_array, ($pos+2), -2 );
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'listingtype' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );
						break;

					default:
						$reviewspattern = _MT_SEF_REVIEWS_PAGE . "[0-9]+";
						$isReviewsPage = eregi($reviewspattern,$url_array[ (count($url_array)-2) ]);

						if ($isReviewsPage) {
							$path_names = array_slice( $url_array, ($pos+2), -2 );
						} else {
							$path_names = array_slice( $url_array, ($pos+2) );
						}
						$link_id = $this->findLinkID( $path_names );
						$QUERY_STRING .= $this->appendURL( 'task', 'viewlink' );
						$QUERY_STRING .= $this->appendURL( 'link_id', $link_id );

						// Handling of $limit & $limitstart
						if ( $isReviewsPage ) {

							// Get the page numner
							$pagenumber = substr( $url_array[ (count($url_array)-2) ], strlen(_MT_SEF_REVIEWS_PAGE) );
							
							$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_reviews') );
							$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_reviews') * ($pagenumber -1)) );

						}
						break;
				}
				
				// Pagination
				$page = $url_array[ (count($url_array)-2) ];
				if (is_numeric($page)) {
					$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($page -1)) );
				}

			# (3) List Alpha
			#   - http://example.com/c/mtree/Game/Action/List_Alpha/[a-z]/[0-9]+
			} elseif( $url_array[ (count($url_array)-4) ] == _MT_SEF_LISTALPHA ) {

				$alpha = $url_array[ (count($url_array)-3) ];
				$page = $url_array[ (count($url_array)-2) ];
				$cat_names = array_slice( $url_array, ($pos+2), -4 );
				$cat_id = $this->findCatID( $cat_names );
				
				$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
				$QUERY_STRING .= $this->appendURL( 'task', 'listalpha' );
				$QUERY_STRING .= $this->appendURL( 'alpha', $alpha );
				$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_links') );
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($page -1)) );

			# (4) List Featured
			#   - http://example.com/c/mtree/Game/Action/Featured_Listing/[0-9]+
			} elseif( $url_array[ (count($url_array)-3) ] == _MT_SEF_FEATUREDLISTING ) {

				$page = $url_array[ (count($url_array)-2) ];
				$cat_names = array_slice( $url_array, ($pos+2), -3 );
				$cat_id = $this->findCatID( $cat_names );
				
				$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
				$QUERY_STRING .= $this->appendURL( 'task', 'listfeatured' );
				$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_links') );
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($page -1)) );
				
			} elseif( $url_array[ (count($url_array)-3) ] == _MT_SEF_VERIFIEDLISTING ) {

				$page = $url_array[ (count($url_array)-2) ];
				$cat_names = array_slice( $url_array, ($pos+2), -3 );
				$cat_id = $this->findCatID( $cat_names );
				
				$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
				$QUERY_STRING .= $this->appendURL( 'task', 'listverified' );
				$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_links') );
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($page -1)) );

			# (5) List New
			#   - http://example.com/c/mtree/Game/Action/New_Listing/[0-9]+
			} elseif( $url_array[ (count($url_array)-3) ] == _MT_SEF_NEWLISTING ) {

				$page = $url_array[ (count($url_array)-2) ];
				$cat_names = array_slice( $url_array, ($pos+2), -3 );
				$cat_id = $this->findCatID( $cat_names );
				$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
				$QUERY_STRING .= $this->appendURL( 'task', 'listnew' );
				//$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_newlisting') );
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($page -1)) );

			# (6) List Owner's listings
			#   - http://example.com/c/mtree/viewowner/Gandalf/1/  _MT_SEF_REVIEWS, _MT_SEF_FAVOURITES,
			} elseif( $url_array[ (count($url_array)-4) ] == _MT_SEF_OWNER || $url_array[ (count($url_array)-4) ] == _MT_SEF_REVIEWS || $url_array[ (count($url_array)-4) ] == _MT_SEF_FAVOURITES ) {
				
				switch( $url_array[ (count($url_array)-4) ] ) {
					case _MT_SEF_OWNER:
						$QUERY_STRING .= $this->appendURL( 'task', 'viewowner' );
						break;
					case _MT_SEF_REVIEWS:
						$QUERY_STRING .= $this->appendURL( 'task', 'viewusersreview' );
						break;
					case _MT_SEF_FAVOURITES:
						$QUERY_STRING .= $this->appendURL( 'task', 'viewusersfav' );
						break;
				}

				$owner_username = $url_array[ (count($url_array)-3) ];
				$owner_username = sefdecode($owner_username);

				$db->setQuery( "SELECT id FROM #__users WHERE username='".$owner_username."' LIMIT 1" );
				$user_id = $db->loadResult();
				$QUERY_STRING .= $this->appendURL( 'user_id', $user_id );

				// Get the page number
				$pagenumber = $url_array[ (count($url_array)-2) ];

				if ( $pagenumber > 0 ) {
					$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_links') );
					$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_links') * ($pagenumber -1)) );
				}

			# (7) Reply Review
			#   - http://example.com/c/mtree/replyreview/123/
			} elseif( $url_array[ (count($url_array)-3) ] == _MT_SEF_REPLYREVIEW ) {

				$rev_id = $url_array[ (count($url_array)-2) ];
				$QUERY_STRING .= $this->appendURL( 'task', 'replyreview' );
				$QUERY_STRING .= $this->appendURL( 'rev_id', $rev_id );

			# (8) Report Review
			#   - http://example.com/c/mtree/reportreview/123/
			} elseif( $url_array[ (count($url_array)-3) ] == _MT_SEF_REPORTREVIEW ) {

				$rev_id = $url_array[ (count($url_array)-2) ];
				$QUERY_STRING .= $this->appendURL( 'task', 'reportreview' );
				$QUERY_STRING .= $this->appendURL( 'rev_id', $rev_id );

			# (9) Search Result
			#		- http://example.com/c/mtree/Search/1/searchword/
			} elseif( $url_array[ (count($url_array)-4) ] == _MT_SEF_SEARCH ) {
				
				$cat_names = array_slice( $url_array, ($pos +2), -4 );
				$cat_id = $this->findCatID( $cat_names );
				$searchword = $url_array[ (count($url_array)-2) ];
				$page = $url_array[ (count($url_array)-3) ];
				
				$QUERY_STRING .= $this->appendURL( 'task', "search" );
				$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
				$QUERY_STRING .= $this->appendURL( 'searchword', sefdecode($searchword) );
				$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_searchresults') );
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_searchresults') * ($page -1)) );

			# (10) Advanced Search Result
			#		- http://example.com/c/mtree/AdvSearchR/1/[search_id]
			} elseif( $url_array[ (count($url_array)-4) ] == _MT_SEF_ADVSEARCH2 ) {

				$search_id = $url_array[ (count($url_array)-2) ];
				$page = $url_array[ (count($url_array)-3) ];

				$QUERY_STRING .= $this->appendURL( 'task', "advsearch2" );
				$QUERY_STRING .= $this->appendURL( 'search_id', $search_id );
				$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_searchresults') );
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_searchresults') * ($page -1)) );
				
			# (10) Tags Search Result
			#		- http://example.com/c/mtree/Search_By/Tags/value/
			} elseif( $url_array[ (count($url_array)-4) ] == _MT_SEF_SEARCHBY ) {
				
				$alias = $url_array[ (count($url_array)-3) ];
				$cf_id = $this->findCfId( sefdecode($alias) );
				
				$value = $url_array[ (count($url_array)-2) ];
				$value = sefdecode($value);
				
				$QUERY_STRING .= $this->appendURL( 'task', "searchby" );
				$QUERY_STRING .= $this->appendURL( 'cf_id', $cf_id );
				$QUERY_STRING .= $this->appendURL( 'value', $value );
				
			# (11) View Image
			#		- http://example.com/c/mtree/image/[img_id]
			} elseif( $url_array[ (count($url_array)-3) ] == _MT_SEF_IMAGE ) {

				$img_id = $url_array[ (count($url_array)-2) ];
				$QUERY_STRING .= $this->appendURL( 'task', 'viewimage' );
				$QUERY_STRING .= $this->appendURL( 'img_id', $img_id );
			
			# (12) RSS
			#		- http://example.com/c/mtree/rss/[updated]
			} elseif( $url_array[ $pos+2 ] == 'rss' ) {

				$type = $url_array[ (count($url_array)-2) ];
				$QUERY_STRING .= $this->appendURL( 'task', 'rss' );
				$QUERY_STRING .= $this->appendURL( 'type', $type );

			# (13) List New
			#   - http://example.com/c/mtree/New_Listing/[0-9]+
			} elseif( $url_array[ (count($url_array)-2) ] == _MT_SEF_NEWLISTING ) {
			
				//echo '<pre>'; print_r($mtconf); echo '</pre>'; die();
			
				$page = $url_array[ (count($url_array)-1) ];
				$cat_id = 0;
				$QUERY_STRING .= $this->appendURL( 'cat_id', $cat_id );
				$QUERY_STRING .= $this->appendURL( 'task', 'listnew' );
				//$QUERY_STRING .= $this->appendURL( 'limit', $mtconf->get('fe_num_of_newlisting') );
				
				$QUERY_STRING .= $this->appendURL( 'limitstart', ($mtconf->get('fe_num_of_new') * ($page -1)) );

			# (14) Tags Search Result
			#		- http://example.com/c/mtree/Search_By/?cf_id=29&value=Accidents_Personal_Injury_Property%20Damage
			} elseif( $url_array[ (count($url_array)-2) ] == _MT_SEF_SEARCHBY ) {

				$QUERY_STRING .= $this->appendURL( 'task', "searchby" );
			}

		}

		// Try to load new $Itemid
		$this->getItemid($QUERY_STRING);

		return $QUERY_STRING;

	}


	/********************************************************
	* Utility Functions
	********************************************************/

	/***
	* Append Categories' Pathway
	*/
	function appendCat( $cat_id ) {
		$sefstring = '';

		$pathWay = new mtPathWay( $cat_id );
		$pathway_ids = $pathWay->getPathWay( $cat_id );
		
		if( !empty($pathway_ids) ) {
			foreach( $pathway_ids AS $id ) {
				if ( _MT_SEF_USE_ALIAS ) {
					$sefstring .= $pathWay->getCatAlias( $id ) . '/';
				} else {
					$sefstring .= sefencode($pathWay->getCatName( $id )) . '/';
				}
			}
		}
		
		// If curreny category is not root, append to sefstring
		if ( _MT_SEF_USE_ALIAS ) {
			$cat_name = $pathWay->getCatAlias($cat_id);
		} else {
			$cat_name = $pathWay->getCatName($cat_id);
		}
		if ( $cat_id > 0 && !empty($cat_name) ) {
			if ( _MT_SEF_USE_ALIAS ) {
				$sefstring .= $cat_name . '/';
			} else {
				$sefstring .= sefencode( $cat_name ) . '/';
			}
		}

		return $sefstring;
	}

	/***
	* Append Listing "filename"
	*/
	function appendListing( $link_name, $link_id, $add_details=false ) {
		$sefstring = '';
		
		if ( ! _MT_SEF_USE_ALIAS ) {
			$link_name = sefencode( $link_name );
		}
		
		if( _MT_SEF_ADD_LINKID ) {
			if( _MT_SEF_ADD_LINKID_ATTACH == 1 ) {
				$sefstring = $link_name . _MT_SEF_ADD_LINKID_SEP . $link_id . '/';
			} else {
				$sefstring = $link_id . _MT_SEF_ADD_LINKID_SEP . $link_name . '/';
			}
		} else {
			$sefstring = $link_name . '/';
		}

		if( $add_details ) {
			$sefstring .= _MT_SEF_DETAILS . '/';
		}

		return $sefstring;
	}

	/***
	* Return value from appendCat + appendListing
	*/
	function appendCatListing( $mtLink, $add_extension=true ) {
		if ( _MT_SEF_USE_ALIAS ) {
			$link_name = $mtLink->alias;
		} else {
			$link_name = $mtLink->link_name;
		}
		return $this->appendCat( $mtLink->cat_id ) . $this->appendListing( $link_name, $mtLink->link_id, false );
	}

	/***
	* Retrieve Cat/Listing's ID from URL string
	*/
	function getID( $type, $string ) {

		$temp = split("&amp;" . $type . "_id=", $string);
		if ( count($temp) >= 2 ) {
			$temp = split("&", $temp[1]);
		} else {
			$temp[0] = 0;
		}
		return intval( $temp[0] );

	}

	/***
	* Return true if a link task matches
	*/
	function listingTaskIs( $task, $string ) {
		return eregi("&amp;task=$task&amp;link_id=",$string);
	}

	/***
	* Return true the task matches
	*/
	function taskIs( $task, $string ) {
		return eregi("&amp;task=$task",$string);
	}

	/***
	* Find Category ID from an array list of names
	* @param array Category name retrieved from SEF Advance URL. 
	*/
	function findCatID( $cat_names ) {
		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}

		if ( count($cat_names) == 0 ) {
			return 0;
		}
		
		if ( _MT_SEF_USE_ALIAS ) {
			$where = "alias = '".$cat_names[ (count($cat_names)-1) ]."'";
		} else {
			$where = "cat_name = '".sefdecode($cat_names[ (count($cat_names)-1) ])."'";
		}

		// (1) 
		// First Attempt will try to search by category name. 
		// If it returns one result, then this is most probably the correct category
		
		$db->setQuery( "SELECT cat_id FROM #__mt_cats WHERE cat_published='1' AND cat_approved='1' && $where" );
		$cat_ids = $db->loadResultArray();
		
		if ( count($cat_ids) == 1 && $cat_ids[0] > 0 ) {

			return $cat_ids[0];
		
		} else {

		// (2)
		// Second attempt will search the category ID by looking from top level to bottom
			$cat_ids = array();

			for( $i=0; $i<count($cat_names); $i++ ) {
				if ( _MT_SEF_USE_ALIAS ) {
					$where = "alias = '".$cat_names[$i]."'";
				} else {
					$where = "cat_name = '".sefdecode($cat_names[$i])."'";
				}

				$sql = "SELECT cat_id FROM #__mt_cats "
					.	"\n WHERE cat_published='1' AND cat_approved='1' && $where";

				if ( $i > 0 ) {
					$sql .= "&& cat_parent='".$cat_ids[$i-1]."' ";
				} else {
					$sql .= "&& cat_parent='0' ";
				}

				$db->setQuery( $sql );
				$cat_ids[$i] = $db->loadResult();

			}
			
			return end($cat_ids);

		}

	}
	
	/***
	* Find Cf ID from tag value
	* @param array Tag value retrieved from SEF Advance URL. 
	*/
	function findCfId($alias) {
		static $tags;
		
		if( !$tags )
		{
			$tags = $this->getTagAliases();
		}
		
		foreach( $tags AS $tag ) {
			if( strtolower(urldecode($tag->alias)) == strtolower($alias) ) {
				return $tag->cf_id;
			}
		}
		return false;
	}
	
	function appendTag($cf_id) {
		static $tags;
		
		if( !$tags )
		{
			$tags = $this->getTagAliases();
		}
		
		if( isset($tags[$cf_id]) ) {
			return $tags[$cf_id]->alias;
		} else {
			return false;
		}
	}

	function getTagAliases() {
		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}
		$db->setQuery( 'SELECT cf_id, caption FROM #__mt_customfields WHERE tag_search = 1 AND published = 1' );
		$tags = $db->loadObjectList('cf_id');
		if( !empty($tags) )
		{
			foreach($tags AS $tag)
			{
				$tags[$tag->cf_id]->alias = urlencode($tag->caption);
			}
		}
		return $tags;
	}

	/***
	* Find Listing ID from an array list of names
	* @param array Path names retrieved from SEF Advance URL - http://example.com/c/mtree/Games/Computer_Games/fluid/
	*																													^________________________^
	*/
	function findLinkID( $path_names ) {
		if (class_exists('JFactory')) {
			$db = &JFactory::getDbo();
		} else {
			global $database;
			$db = $database;
		}

		// (1) 
		// First Attempt will try to search by listing name. 
		// If it returns one result, then this is most probably the correct listing
		
		$link_name = $path_names[ (count($path_names)-1) ];
		$link_name = urldecode( $link_name );

		if( _MT_SEF_ADD_LINKID ) {
			if( _MT_SEF_ADD_LINKID_ATTACH == 1 ) {
				// suffix 

				$link_ids[0] = substr( $link_name, (strrpos( $link_name, _MT_SEF_ADD_LINKID_SEP ) + strlen(_MT_SEF_ADD_LINKID_SEP)), ( strlen($link_name)  - strrpos( $link_name, _MT_SEF_ADD_LINKID_SEP ) ) );
				
			} else {
				// prefix
				
				$link_ids[0] = substr( $link_name, 0, strpos( $link_name, _MT_SEF_ADD_LINKID_SEP ) );

			}
			
			if( is_numeric($link_ids[0]) ) {
				return $link_ids[0];
			}

		}
		$link_name = sefdecode( $path_names[ (count($path_names)-1) ]);

		$db->setQuery( "SELECT link_id FROM #__mt_links WHERE link_published='1' AND link_approved='1' && link_name ='".$link_name."' " );
		$link_ids = $db->loadResultArray();

		if ( count($link_ids) == 1 && $link_ids[0] > 0 ) {

			return $link_ids[0];
		
		} else {

		// (2)
		// Second attempt will look for the category ID and then pinpoint the listing ID
			
			$cat_id = $this->findCatID( array_slice($path_names, 0, -1) );
			
			$db->setQuery( "SELECT l.link_id FROM #__mt_links AS l, #__mt_cl AS cl WHERE link_published='1' AND link_approved='1' AND cl.cat_id = '".$cat_id."' AND link_name ='".$link_name."' AND l.link_id = cl.link_id LIMIT 1" );
			
			return $db->loadResult();

		}
	}

	/***
	* Routine function to assign values to $_GET, $_REQUEST and $QUERY_STRING
	*/
	function appendURL( $var, $value ) {
		$_GET[$var] = $value;
		$_REQUEST[$var] = $value;
		return "&".$var."=".$value;
	}

	/***
	* Routine function to restrive $limit & $limitvalue from query string
	*
	* @param int A referenced $limit - number of results shown per page
	* @param int A referenced $limitstart - the record number to start display
	* @param string Query string
	*/
	function getLimits( &$limit, &$limitstart, $string ) {
		// limit
		$temp = split("&amp;limit=", $string);
		if (count($temp) >= 2) {
			$temp = split("&", $temp[1]);
			$limit = $temp[0];
		} else {
			$limit = '';
		}

		// limitstart
		$temp = split("&amp;limitstart=", $string);
		if (count($temp) >= 2) {
			$temp = split("&", $temp[1]);
			$limitstart = $temp[0];
		} else {
			$limitstart = '';
		}

		if ( $limit <> '' && $limitstart <> '' ) {
			return true;
		} else {
			return false;
		}

	}

	/***
	* Try to find the page number from virtual directory - http://example.com/c/mtree/My_Listing/Page3.html
	*
	* @param array $url_array The SEF advance URL split in arrays (first custom virtual directory beginning at $pos+1)
	* @return int Page number
	*/
	function getPageNumber( $url_array) {
		$pagepattern = _MT_SEF_CATEGORY_PAGE . "[0-9]+";
		$pagenumber = 0;

		if ( eregi($pagepattern,$url_array[ (count($url_array)-2) ]) ) {

			// Get the page number
			$pagenumber = substr( $url_array[ (count($url_array)-2) ], strlen(_MT_SEF_CATEGORY_PAGE));

		}

		return $pagenumber;

	}
	
	function append_ordering_and_limits($string) {
		global $sef_request_uri;
		$sefstring = '';
		
		// if query string already present in request save it
		if (stristr($sef_request_uri,'?')) {
			$temp = explode('?',$sef_request_uri);
			$query_string = $temp[count($temp)-1];
			$pairs = explode('&',$query_string);
			$query_array = array();
			foreach($pairs as $pair) {
				list($name,$value) = explode('=',$pair);
				if ($name=='Field') {
					$query_array[] = "$name=$value";
				}
			}
			if (count($query_array)>0) {
				$query_string = implode('&',$query_array);
				$sefstring .= '?'.$query_string;
			}
		}
		
		// ordering & ordering_dir
		$temp = explode("&amp;ordering=", $string);
		if (!empty($temp[1])) {
			$temp = explode("&", $temp[1]);
			$ordering = $temp[0];
			$sefstring .= $this->append_query_string($sefstring, 'ordering', $ordering);
		}
		
		$temp = explode("&amp;ordering_dir=", $string);
		if (!empty($temp[1])) {
			$temp = explode("&", $temp[1]);
			$ordering_dir = $temp[0];
			$sefstring .= $this->append_query_string($sefstring, 'ordering_dir', $ordering_dir);
		}
		
		// limit & limitstart
		$temp = explode('&amp;limit=', $string);
		if(!empty($temp[1])) {
			$temp = explode('&', $temp[1]);
			$limit = $temp[0];
			$sefstring .= $limit!='' ? $this->append_query_string($sefstring, 'limit', $limit) : '';
		}

		$temp = explode('&amp;limitstart=', $string);
		if(!empty($temp[1])) {
			$temp = explode('&', $temp[1]);
			$limitstart = $temp[0];
			$sefstring .= $limitstart!='' ? $this->append_query_string($sefstring, 'limitstart', $limitstart) : '';
		}
		
		return $sefstring;
	}
	
	function append_query_string($string, $name, $value) {
		if (stristr($string, '?')) {
			$append = '&';
		} else {
			$append = '?';
		}
		$append .= "$name=$value";
		return $append;
	}

	function getItemid($query_string) {
		if (!$query_string) {
			return;
		}
		$query_string = str_replace('task', 'view', $query_string);
		$string = 'index.php?option=com_mtree' . $query_string;
		$db = JFactory::getDbo();
		$query = 'SELECT id FROM #__menu WHERE link='.$db->q($string);
		$db->setQuery($query);
		if ($result = $db->loadResult()) {
			$_GET['Itemid'] = $_REQUEST['Itemid'] = (int) $result;
		}
	}

}
?>